# -*- coding: utf-8 -*-
from ..utils.ValidUtil import ValidUtil
from .ValidateResult import ValidateResult

"""
 * 验证器基类
 *<B>说明：</B>
 *<pre>
 *  略
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""

class Validator(object):

    def __init__(self,attrs = {}):

        self.validation = None

        # 验证的数据
        # <B> 说明： </B>
        # <pre>
        # 或
        # </pre>
        self.attributes =  {};

        # 验证器名称
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.validatorName = ''

        # 验证错误提示信息
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.message = ''

        # 验证值为空时是否调用验证
        # <B> 说明： </B>
        # <pre>
        # true 表示　值为空时,不验证,false 表示值为空，继续验证
        # </pre>
        self.skipOnEmpty = True

        # 验证是否空方法
        # <B> 说明： </B>
        # <pre>
        # 一般函数或[$this,'isEmpty']
        # 比例利用php 的empty 函数验证空值
        # </pre>
        self.emptyFunc = None

        # 结果是否非
        # <B> 说明： </B>
        # <pre>
        # 如:!$result
        # </pre>
        self.non = False

        # 消息替换参数
        # <B> 说明： </B>
        # <pre>
        # 在验证validateValue中给设置params值
        #  {'min'=>'1','max'=>'10',..}
        # </pre>
        self.params = {}

        if attrs:
            ValidUtil.setAttrs(self,attrs)

    def setAttributes(self,attributes):

        self.attributes = attributes;

        return ;

    def getAttribute(self,attrName:str):

        attrNameList = attrName.split('.')
        attributeName = attrNameList.pop(0);
        if isinstance(self.attributes,dict):
            value = self.attributes[attributeName]
        else:
            value = getattr(self.attributes,attributeName)

        for attr in attrNameList:
            value = value.get(attr,None)

        return value;

    def setValidation(self,validation):

        self.validation = validation;

        return ;


    # 判断给定的值是否为空
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def isEmpty(self,value):

        if self.emptyFunc:
            return self.emptyFunc(value)
        else:
            if value:
                return False
            else:
                return True


    # 验证方法接口
    # <B> 说明： </B>
    # <pre>
    # 子类必须实现此方法
    # </pre>
    # param value 验证值
    # param name 验证名称
    def validateValue(self,value,name):

        return True

    # 验证规则
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # param rule
    def validateRule(self,rule):

        attrs = rule.getAttrs();
        validateResultList = [];
        for attrName in attrs:
            value = self.getAttribute(attrName)
            validateResultList.append(self.validate(value, attrName))

        if False in validateResultList:
            return ValidateResult(False, self.getMessage(), {})
        else:
            return ValidateResult(True)

    # 是否跳过验证
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # param values 属性值列表
    def isSkip(self,value):

        if self.skipOnEmpty and self.isEmpty(value):
            return True
        else:
            return False

    # 验证单值接口
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # param values 属性值列表
    # return boolean
    def validate(self,value,name = None):

        if self.isSkip(value):
            return True

        result = self.validateValue(value,name)
        if  self.non is True:
            result = bool(1-result)

        return result

    def getMessage(self):

        msg = self.message;
        if not msg:
            # 获取默认消息
            msg = self.validation.getDefaultMessage(self.validatorName);

        msg = self.formatMessage(msg,self.params)

        return msg;

    def formatMessage(self,message,params = {}):

        if not params:
            return message

        replaceList = {}
        paramsType = type(params)

        if paramsType is list:
            for index in range(len(params)):
                replaceList['{' + str(index) + '}'] = params[index]
        else:
            for key in params:
                replaceList['{' + key + "}"] = str(params[key])


        return ValidUtil.replaceAll(message,replaceList)

    def addParam(self,key,value):

        self.params[key] = value

        return ;

    def addParams(self,params = {}):

        self.params.update(params)

        return True



# -*- coding: utf-8 -*-
from ..base.Validator import Validator
import re
"""
 * 手机号验证器
 *<B>说明：</B>
 *<pre>
 * 11 位手机验证格式
 * 规则格式:
 * ['attrs',[['tel']],{'message'=>'你输入的手机号格式错误！'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class TelValidator(Validator):


    def __init__(self,attrs):
        self.pattern = r'^1[0-9]{1}\d{9}$'
        super().__init__(attrs)

    def validateValue(self,value,name = None):

        if re.match(self.pattern,str(value)) is None:
            return False
        else:
            return True




# -*- coding: utf-8 -*-
from ..base.Validator import Validator
import re
"""
 * 货币验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['currency']],{'message'=>'请输入一个合法的货币类型数值！'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class CurrencyValidator(Validator):

    def __init__(self,attrs):
        self.pattern = r'^[-\+]?(([1-9]{1}\d*)|([0]{1}))(\.(\d){1,{point}})?$'
        self.decimalPoint = 2;
        super().__init__(attrs)

    def validateValue(self,value,name = None):
        value = str(value)
        self.addParams('{point}',self.decimalPoint)
        if value.find(self.pattern) != -1:
            pattern = self.pattern.replace("{point}",'(' + "|".join(self.decimalPoint) + ')')
        else:
            pattern = self.pattern

        if re.match(pattern,value) is None:
            return False
        else:
            return True




# -*- coding: utf-8 -*-
from ..base.Validator import Validator
from ..utils.ValidUtil import ValidUtil
"""
 * 字段值比较验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['range',{number':6,'operator':'gt'}]],{'message'=>'请输入一个大于15的数值'}]
 * operator:大于(gt),大于等于(egt),小于(lt),小于等于(elt)
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class CompareFieldValidator(Validator):

    def __init__(self,attrs):

        self.field = '' # 比较的字段startdate,userid
        self.comparetype = 'int' # 比较类型int,str,date
        self.operator = ''# 比较的操作符 gt edt lit 等等
        self.operatorList = ['gt','egt','lt','elt','eq','>','>=','<','<=','=']

        super().__init__(attrs)

    def getFormatValue(self,value):

        if self.comparetype == 'int':
            return float(value)
        elif self.comparetype == 'date':
            return ValidUtil.strtotime(value);
        elif self.comparetype == 'str':
            return str(value)
        else:
            return int(value)

    def validateValue(self,value,name = None):

        self.addParam('field2',name);
        self.addParam('field1', self.field);

        comparefieldValue = self.getFormatValue(self.getAttribute(self.field))

        if self.operator not in self.operatorList:
            return False

        result = False
        validValue = self.getFormatValue(value)

        if self.operator == 'gt' or self.operator == '>':
            # 大于
            result = (validValue > comparefieldValue)
        elif self.operator == 'egt' or self.operator == '>=':
            # 大于等于
            result = (validValue >= comparefieldValue)
        elif self.operator == 'lt' or self.operator == '<':
            # 小于
            result = (validValue < comparefieldValue)
        elif self.operator == 'elt' or self.operator == '<=':
            # 小于等于
            result = (validValue <= comparefieldValue)
        elif self.operator == 'eq' or self.operator == '=':
            # 小于等于
            result = (validValue == comparefieldValue)
        else:
            pass

        return result


# -*- coding: utf-8 -*-
import importlib
import time,datetime

"""
 * 类帮助类
 *<B>说明：</B>
 *<pre>
 *  提供基本类操作,获取属性值,设置属性等等
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class ValidUtil:

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def getAttrs(cls, object):

        attrs = object.__dict__;
        attrDict = {};
        for attr in attrs:
            if not attr.startswith("_"):
                attrDict[attr] = getattr(object, attr)

        return attrDict

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def getAttrValues(cls, object,attrNames = []):

        attrDict = {};
        for attrName in attrNames:
            attrDict[attrName] = getattr(object, attrName)

        return attrDict

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def setAttrs(cls,object,attrDict = {}):

        for attr in attrDict:
            setattr(object, attr, attrDict[attr])

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def getModuleMeta(cls,clazz):

        if isinstance(clazz,str):
            packageClass = clazz.rsplit('.', 1)
            packageMeta = importlib.import_module(packageClass[0])
            return  getattr(packageMeta, packageClass[1])
        else:
            return clazz

    @classmethod
    def replaceAll(cls, str='', replaceList={}):

        newStr = '' + str
        for find in replaceList:
            replace = replaceList[find];
            newStr = newStr.replace(find, replace)

        return newStr

    @classmethod
    def listToStr(cls, values = []):

        for i in range(0, values.__len__()):
            values[i] = str(values[i])

        return values

    # 字符串日期转时间戳
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def strtotime(self,strtime:str):

        datetimelist = strtime.split(' ')
        datelist = datetimelist[0].split('-');

        if len(datetimelist) >= 2:
            timelist =  datetimelist[1].split(":")
        else:
            timelist = [];

        dt_datetime_list = [0,0,0,0,0,0]

        for i in range(len(datelist)):
            dt_datetime_list[i] = int(datelist[i]);

        for i in range(len(timelist)):
            index = i + 3
            dt_datetime_list[index] = int(timelist[i]);

        year,month,day,hour, minute, second = dt_datetime_list;
        dtime = datetime.datetime(year,month,day,hour,minute,second)

        return int(time.mktime(dtime.timetuple()))




# -*- coding: utf-8 -*-
from ..base.Validator import Validator
from ..utils.ValidUtil import ValidUtil
"""
 * 日期范围验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['rangedate',{min':6,'max':16}]],{'message'=>'请输入一个长度为合法的6-16的字符串'}]
 * ['attrs',[['rangedate',{min':6}]],{'message'=>'请输入一个长度大于等于6的字符串'}]
 * ['attrs',[['rangedate',{'max':16}]],{'message'=>'请输入一个长度小于等于16的字符串'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class RangeDateValidator(Validator):

    def __init__(self,attrs):
        self.min = None
        self.max = None

        super().__init__(attrs)

    def validateValue(self,value,name = None):

        validDatetime = ValidUtil.strtotime(value);
        result = True

        if self.min is not None:
            minctime = ValidUtil.strtotime(self.min);
            if validDatetime <= minctime:
                result = False

        if self.max is not None:
            maxctime = ValidUtil.strtotime(self.max);
            if validDatetime > maxctime:
                result = False

        if result:
            return True
        else:
            self.addParams({
                'min': self.min,
                'max': self.max
            })

        return result;




# -*- coding: utf-8 -*-
from ..base.Validator import Validator
import re
"""
 * 比较值验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['range',{number':6,'operator':'gt'}]],{'message'=>'请输入一个大于15的数值'}]
 * operator:大于(gt),大于等于(egt),小于(lt),小于等于(elt)
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class CompareValidator(Validator):

    def __init__(self,attrs):
        self.number = 0
        self.operator = ''
        self.operatorList = ['gt','egt','lt','elt','eq','>','>=','<','<=','=']

        super().__init__(attrs)

    def validateValue(self,value,name = None):
        self.addParam('number',self.number);

        reg = r'^[-\+]?\d+(\.\d+)?$/'
        if re.match(reg,value) is None:
            return False

        if self.operator not in self.operatorList:
            return False

        result = False
        value = float(value)

        if self.operator == 'gt' or self.operator == '>':
            # 大于
            result = (value > self.number)
        elif self.operator == 'egt' or self.operator == '>=':
            # 大于等于
            result = (value >= self.number)
        elif self.operator == 'lt' or self.operator == '<':
            # 小于
            result = (value < self.number)
        elif self.operator == 'elt' or self.operator == '<=':
            # 小于等于
            result = (value <= self.number)
        elif self.operator == 'eq' or self.operator == '=':
            # 小于等于
            result = (value == self.number)
        else:
            pass

        return result


# -*- coding: utf-8 -*-
from ..base.Validator import Validator
from ..utils.ValidUtil import ValidUtil

"""
 * in 集合验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['inlist',{'numbers':[1,2,3]}]],{'message'=>'输入的值必须为1,2,3,4'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class InValidator(Validator):

    def __init__(self,attrs):
        self.numbers = []

        super().__init__(attrs)
        self.numbers = ValidUtil.listToStr(self.numbers)

    def validateValue(self,value,name = None):

        self.addParam('numbers', ','.join(self.numbers));
        if str(value) in self.numbers:
            return True
        else:
            return False




# -*- coding: utf-8 -*-

from ..utils.ValidUtil import ValidUtil

"""
 * 验证规则
 *<B>说明：</B>
 *<pre>
 *  略
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class Rule(object):

    # 默认场景
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    DEFUALT_SCENE = 'default';

    def __init__(self,ruleConf = []):

        # 验证的属性名
        # <B> 说明： </B>
        # <pre>
        # app 应用级别，forever 永远不失效
        # </pre>
        self.attrs = {};

        # 验证器列表
        # <B> 说明： </B>
        # <pre>
        # [
        #    ['boolean','skipOnEmpty'=>false]
        # ]
        # </pre>
        self.validators = []

        # 当验证错误是是否继续验证
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.goon = True;

        # 错误消息
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.message = '';

        # 场景
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.on = '';

        # 满足条件
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.when = None;

        attrConf = [];
        attrConf = attrConf + ruleConf
        attr = attrConf[0]
        attrList = [];
        if type(attr) is not list:
            attrList.append(attr)
        else:
            attrList = attr;

        self.attrs = attrList

        validator = attrConf[1]
        validatorList = [];
        if type(validator) is not list:
            validatorList.append(attr)
        else:
            validatorList = validator;

        self.validators = validatorList
        if len(attrConf) >= 3:
            attrs = attrConf[2]
            ValidUtil.setAttrs(self,attrs)


    def getMessage(self):

        return self.message

    def getValidators(self):

        return self.validators;

    def getGoon(self):

        return self.goon;

    def getAttrs(self):

        return self.attrs

    # 是否有效规则
    # <B> 说明： </B>
    # <pre>
    # 根据场景过滤出适合符合传入场景的rule
    # </pre>
    def isActive(self,scenes = [],attributes = {}):

        if self.on and (self.on != self.DEFUALT_SCENE or self.on not in scenes):
            return False

        if self.when and self.when(attributes) is False:
            return False;

        return True





# -*- coding: utf-8 -*-
from ..base.Validator import Validator
import re

"""
 * 邮箱验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['email']],{'message'=>'你输入的邮箱格式错误！'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class EmailValidator(Validator):


    def __init__(self,attrs):
        self.pattern = r"^[a-zA-Z0-9!#$%&'*+\\/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&'*+\\/=?^_`{|}~-]+)*@(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?\.)+[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?$"

        self.fullPattern = r"^[^@]*<[a-zA-Z0-9!#$%&'*+\\/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&'*+\\/=?^_`{|}~-]+)*@(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?\.)+[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?>$"

        self.allowName = False;

        super().__init__(attrs)

    def validateValue(self,value,name = None):

        value = str(value)

        if re.match(self.pattern,value) is not None or (self.allowName or re.match(self.fullPattern,value) is not None):
            return True
        else:
            return False



# -*- coding: utf-8 -*-
from ..base.Validator import Validator

"""
 * 布尔值类型验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['boolean']],{'message'=>'你输入的值非布尔值类型'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class BooleanValidator(Validator):

    def validateValue(self,value,name = None):

        if isinstance(value,bool):
            return True
        else:
            return False




# -*- coding: utf-8 -*-
from ..base.Validator import Validator

"""
 * 相等验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['eq',{number':6}]],{'message'=>'请输入一个等于6的值'}]
 * operator:大于(gt),大于等于(egt),小于(lt),小于等于(elt)
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class EqualValidator(Validator):

    def __init__(self,attrs):
        self.number = 0
        self.operator = '=='
        self.operatorList = ['==','==']

        super().__init__(attrs)

    def validateValue(self,value,name = None):
        self.addParam('number',self.number);

        if self.operator not in self.operatorList:
            return False

        result = False

        if self.operator == '==':
            # 大于
            result = (str(value) == str(self.number))
        elif self.operator == '===':
            # 大于等于
            result = (value is self.number)
        else:
            pass

        return result


# -*- coding: utf-8 -*-
from ..base.Validator import Validator
import time
"""
 * 日期验证器
 *<B>说明：</B>
 *<pre>
 * 规则格式:
 * ['attrs',[['date',{'format':'Y-m-d'}]],{'message'=>'你输入的日期格式错误,正确日期格式为:{format}'}]
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class DateValidator(Validator):

    def __init__(self,attrs):
        self.format = 'Y-m-d'
        super().__init__(attrs)


    def validateValue(self,value,name = None):

        try:
            time.strptime(value, self.format)
            self.addParam('format',self.format)
            return True
        except:
            return False